@@ -1,3 +1,4 @@
+use std::borrow::Cow;
 use std::collections::{BTreeMap, Bound};
 use std::ffi::OsStr;
 use std::fmt::Display;
@@ -123,9 +124,12 @@ impl<'de> Deserialize<'de> for VerbatimPackageName {
     where
         D: Deserializer<'de>,
     {
-        let given = String::deserialize(deserializer)?;
+        let given = <Cow<'_, str>>::deserialize(deserializer)?;
         let normalized = PackageName::from_str(&given).map_err(serde::de::Error::custom)?;
-        Ok(Self { given, normalized })
+        Ok(Self {
+            given: given.to_string(),
+            normalized,
+        })
     }
 }
 
