@@ -15,6 +15,7 @@ pub use name_specifiers::*;
 pub use overrides::*;
 pub use package_options::*;
 pub use project_build_backend::*;
+pub use proxy_url::*;
 pub use required_version::*;
 pub use sources::*;
 pub use target_triple::*;
@@ -40,6 +41,7 @@ mod name_specifiers;
 mod overrides;
 mod package_options;
 mod project_build_backend;
+mod proxy_url;
 mod required_version;
 mod sources;
 mod target_triple;
