@@ -15,6 +15,7 @@ use uv_distribution_types::{
 };
 use uv_git_types::{GitLfs, GitOid};
 use uv_normalize::PackageName;
+use uv_pep440::Version;
 use uv_platform_tags::{IncompatibleTag, TagCompatibility, Tags};
 use uv_pypi_types::{DirInfo, DirectUrl, VcsInfo, VcsKind};
 
@@ -36,6 +37,7 @@ impl RequirementSatisfaction {
         name: &PackageName,
         distribution: &InstalledDist,
         source: &RequirementSource,
+        version: Option<&Version>,
         installation: InstallationStrategy,
         tags: &Tags,
         config_settings: &ConfigSettings,
@@ -358,6 +360,21 @@ impl RequirementSatisfaction {
             }
         }
 
+        // If a resolved version is provided, check that it matches the installed version.
+        // This is needed for sources that don't include explicit version specifiers (e.g.,
+        // directory dependencies with dynamic versioning), where the resolver may have determined
+        // a new version should be installed.
+        if let Some(version) = version {
+            if distribution.version() != version {
+                debug!(
+                    "Installed version does not match resolved version for {name}: {} vs. {}",
+                    distribution.version(),
+                    version
+                );
+                return Self::OutOfDate;
+            }
+        }
+
         // Otherwise, assume the requirement is up-to-date.
         Self::Satisfied
     }
